import { randomUUID } from 'node:crypto'
import { Collection, Entity, OneToMany, PrimaryKey, Property } from '@mikro-orm/core'
import { BaseEntity } from '#application/bases/baseEntity'
import { SpriteAction } from './spriteAction'
import { CharacterType } from './characterType'
import { CharacterHair } from './characterHair'
import { Item } from './item'
import { UUID } from '#application/types'

@Entity()
export class Sprite extends BaseEntity {
  @PrimaryKey()
  id = randomUUID()

  @Property()
  name!: string

  @Property()
  createdAt = new Date()

  @Property()
  updatedAt = new Date()

  @OneToMany(() => SpriteAction, (action) => action.sprite)
  spriteActions = new Collection<SpriteAction>(this)

  @OneToMany(() => CharacterType, (type) => type.sprite)
  characterTypes = new Collection<CharacterType>(this)

  @OneToMany(() => CharacterHair, (hair) => hair.sprite)
  characterHairs = new Collection<CharacterHair>(this)

  @OneToMany(() => Item, (item) => item.sprite)
  items = new Collection<Item>(this)

  setId(id: UUID) {
    this.id = id
    return this
  }

  getId() {
    return this.id
  }

  setName(name: string) {
    this.name = name
    return this
  }

  getName() {
    return this.name
  }

  setCreatedAt(createdAt: Date) {
    this.createdAt = createdAt
    return this
  }

  getCreatedAt() {
    return this.createdAt
  }

  setUpdatedAt(updatedAt: Date) {
    this.updatedAt = updatedAt
    return this
  }

  getUpdatedAt() {
    return this.updatedAt
  }
}
