import { appLogger } from '#application/logger'
import { BaseRepository } from '#application/bases/baseRepository'
import { Character } from '#entities/character'

class CharacterRepository extends BaseRepository {
  async getByUserId(userId: number): Promise<Character[]> {
    try {
      const repository = this.em.getRepository(Character)
      return await repository.find({
        user: userId
      })
    } catch (error: any) {
      appLogger.error(`Failed to get character by user ID: ${error instanceof Error ? error.message : String(error)}`)
      return []
    }
  }

  async getByUserAndId(userId: number, characterId: number): Promise<Character | null> {
    try {
      const repository = this.em.getRepository(Character)
      return await repository.findOne({
        user: userId,
        id: characterId
      })
    } catch (error: any) {
      appLogger.error(`Failed to get character by user ID and character ID: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getById(id: number): Promise<Character | null> {
    try {
      const repository = this.em.getRepository(Character)
      return await repository.findOne({ id })
    } catch (error: any) {
      appLogger.error(`Failed to get character by ID: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getByName(name: string): Promise<Character | null> {
    try {
      const repository = this.em.getRepository(Character)
      return await repository.findOne({ name })
    } catch (error: any) {
      appLogger.error(`Failed to get character by name: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }
}

export default new CharacterRepository()
