import { appLogger } from '#application/logger'
import { BaseRepository } from '#application/bases/baseRepository'
import { Item } from '#entities/item'

class ItemRepository extends BaseRepository {
  async getById(id: string): Promise<any> {
    try {
      const repository = this.em.getRepository(Item)
      return await repository.findOne({ id })
    } catch (error: any) {
      appLogger.error(`Failed to get item by ID: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getByIds(ids: string[]): Promise<any> {
    try {
      const repository = this.em.getRepository(Item)
      return await repository.find({
        id: ids
      })
    } catch (error: any) {
      appLogger.error(`Failed to get items by IDs: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }

  async getAll(): Promise<any> {
    try {
      const repository = this.em.getRepository(Item)
      return await repository.findAll()
    } catch (error: any) {
      appLogger.error(`Failed to get all items: ${error instanceof Error ? error.message : String(error)}`)
      return null
    }
  }
}

export default new ItemRepository()
