import { appLogger } from '#application/logger'
import { BaseRepository } from '#application/bases/baseRepository' // Import the global Prisma instance
import { PasswordResetToken } from '#entities/passwordResetToken'

class PasswordResetTokenRepository extends BaseRepository {
  async getById(id: number): Promise<any> {
    try {
      const repository = this.em.getRepository(PasswordResetToken)
      return await repository.findOne({ id })
    } catch (error: any) {
      // Handle error
      appLogger.error(`Failed to get password reset token by ID: ${error instanceof Error ? error.message : String(error)}`)
    }
  }

  async getByUserId(userId: number): Promise<any> {
    try {
      const repository = this.em.getRepository(PasswordResetToken)
      return await repository.findOne({
        user: userId
      })
    } catch (error: any) {
      // Handle error
      appLogger.error(`Failed to get password reset token by user ID: ${error instanceof Error ? error.message : String(error)}`)
    }
  }

  async getByToken(token: string): Promise<any> {
    try {
      const repository = this.em.getRepository(PasswordResetToken)
      return await repository.findOne({ token })
    } catch (error: any) {
      // Handle error
      appLogger.error(`Failed to get password reset token by token: ${error instanceof Error ? error.message : String(error)}`)
    }
  }
}

export default new PasswordResetTokenRepository()
