import { gameLogger } from '#application/logger'
import WorldRepository from '#repositories/worldRepository'
import { World } from '#entities/world'

class WorldService {
  async update(worldData: Partial<World>): Promise<boolean> {
    try {
      let world = await WorldRepository.getFirst()
      if (!world) {
        world = new World()
      }

      world.setDate(worldData.date || new Date())

      if (worldData.isRainEnabled) {
        world.setIsRainEnabled(worldData.isRainEnabled)
      }
      if (worldData.rainPercentage) {
        world.setRainPercentage(worldData.rainPercentage)
      }
      if (worldData.isFogEnabled) {
        world.setIsFogEnabled(worldData.isFogEnabled)
      }
      if (worldData.fogDensity) {
        world.setFogDensity(worldData.fogDensity)
      }

      await world.save()

      return true
    } catch (error: any) {
      gameLogger.error(`Failed to update world: ${error instanceof Error ? error.message : String(error)}`)
      return false
    }
  }
}

export default new WorldService()
